/*
 * 29-04-2016 LOCO Wi-Fi
 *  server_ip is the IP address of the ESP8266 module, will be 
 *  printed to Serial when the module is connected.
 */

#include <ESP8266WiFi.h>
// WLAN Default config:
IPAddress Ip(192, 168, 0, 111); //default IP
IPAddress Gw(192, 168, 0, 1); //default Gateway
IPAddress Sb(255, 255, 255, 0); //default Subnet
const char* ssid = "Z21_6666";
const char* password = "54448049";

// Create an instance of the server
// specify the port to listen on as an argument 80
WiFiServer server(4711);

// Управляющие пины, где wire номер = номеру провода на схеме светофора Viessmann H0 Licht-Ausfahrsignal Art.Nr  4013:
#define WIRE1_PIN  4  // D2
#define WIRE2_PIN  5  // D1
#define WIRE3_PIN  14 // D5
#define WIRE4_PIN  12 // D6

//---
byte speed = 0;
bool bool_dir = false;
bool bool_f0 = false;
bool bool_f1 = false;
bool bool_f2 = false;
bool bool_f3 = false;

// FB part:
//--------------------------------------------------------------------------------------------
 boolean flag0 = false;    // метка о том, что препятствие уже посчитали
 byte last_data = 0; 
 byte data = 0;            // выходное значение

// Функция установки диодов во ВКЛ/ВЫКЛ
void set_lights() {
  // HIGH = диод не горит, LOW = диод горит
  if (bool_f0 == true) { digitalWrite(WIRE1_PIN, LOW); } else { digitalWrite(WIRE1_PIN, HIGH); }
  if (bool_f1 == true) { digitalWrite(WIRE2_PIN, LOW); } else { digitalWrite(WIRE2_PIN, HIGH); }
  if (bool_f2 == true) { digitalWrite(WIRE3_PIN, LOW); } else { digitalWrite(WIRE3_PIN, HIGH); }
  if (bool_f3 == true) { digitalWrite(WIRE4_PIN, LOW); } else { digitalWrite(WIRE4_PIN, HIGH); }
}

void setup() {
  Serial.begin(115200);
  delay(10);

  // Connect to WiFi network
  Serial.println();
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);
  WiFi.config(Ip,Gw,Sb); 
  WiFi.begin(ssid, password);
  
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi connected");
  
  // Start the server
  server.begin();
  Serial.println("Server started");

  // Print the IP address
  Serial.println(WiFi.localIP());

  //Выставляем управляющие пины на вывод:
  pinMode(WIRE1_PIN, OUTPUT); pinMode(WIRE2_PIN, OUTPUT); pinMode(WIRE3_PIN, OUTPUT); pinMode(WIRE4_PIN, OUTPUT);

  //Выставляем их в HIGН, что бы при запуске светофор не горел. Так как они подключены к общему плюсу и минусом к выводам, то управляющая логика будет обратной:
  // HIGH = OFF, LOW = ON:
  digitalWrite(WIRE1_PIN, HIGH); digitalWrite(WIRE2_PIN, HIGH); digitalWrite(WIRE3_PIN, HIGH); digitalWrite(WIRE4_PIN, HIGH);
   
}

void loop() {
  // Check if a client has connected
  WiFiClient client = server.available();
  if (!client) {
    return;
  }
 
  Serial.println("new client");  // Wait until the client sends some data
  
  while(!client.available()){

// датчик 0
  if ((!digitalRead(13)) && (flag0 == false)){    // сравниваем. Если меньше И препятствие еще не зафиксировано
     data = (data + 1);                          // 
     flag0 = true;                               // ставим метку, что препятствие посчитано
     }
  if ((digitalRead(13)) && (flag0 == true)){     // опять сравниваем. Если больше И препятствие уже было зафиксировано
     data = (data - 1);
     flag0 = false;                              // сбрасываем метку, т.е. следующее препятствие будет снова посчитано
     }

  if (data != last_data){    //Если состояние входа изменилось, расскажем
    if (data == 1){ 
      client.println("<fb id=\"fb-5\" state=\"true\" fbtype=\"0\"/>");
    } else { 
      client.println("<fb id=\"fb-5\" state=\"false\" fbtype=\"0\"/>");
    }
    last_data = data; // запоминаем текущие состояния
    }

    delay(1);
  }

  //--- chtenie stroki
 String str = client.readStringUntil('/');

 client.stop();
 Serial.println("client disonnected");

 //--- obrabotka
 char *clientline = &str[0]; //string -> char

 // Print it out for debugging
 Serial.println("------------------------------------");
 Serial.print("clientline:");
 Serial.println(clientline);

 // parse commands
 if (strstr(clientline, "<fn ") != 0) {
   if (strstr(clientline, "V200") != 0) {
     Serial.println("Fn V200");
     char *get_arg;
     get_arg = strstr(clientline, "V200");
     //f0
     bool_f0 = ((strstr(get_arg, "f0=")+4)[0] == 't');
     Serial.print("F-0=");
     Serial.println(bool_f0);
     //f1
     bool_f1 = ((strstr(get_arg, "f1=")+4)[0] == 't');
     Serial.print("F-1=");               
     Serial.println(bool_f1);
     //f2
     bool_f2 = ((strstr(get_arg, "f2=")+4)[0] == 't');
     Serial.print("F-2=");               
     Serial.println(bool_f2);
     //f3
     bool_f3 = ((strstr(get_arg, "f3=")+4)[0] == 't');
     Serial.print("F-3=");
     Serial.println(bool_f3);
     // ispolnenie
     set_lights();
   } 
 } 
 else if (strstr(clientline, "<lc ") != 0) {
        char *locname;
        locname = clientline + 4;
        Serial.print("Locname:");
        Serial.println(locname);
          // parse commands       
          if (strstr(locname, "V200") != 0) {
            Serial.println("play track V200");
            char *get_arg;
            get_arg = strstr(locname, "V=");
            speed = atol(get_arg + 3);
            Serial.print("Speed=");
            Serial.println(speed, DEC);
            //DIR
            bool_dir = ((strstr(get_arg, "dir=")+5)[0] == 't');
            Serial.print("DIR=");
            Serial.println(bool_dir);
            //f0
            bool_f0 = ((strstr(get_arg, "f0=")+4)[0] == 't');
            Serial.print("F-0=");
            Serial.println(bool_f0);
            //f1
            bool_f1 = ((strstr(get_arg, "f1=")+4)[0] == 't');
            Serial.print("F-1=");               
            Serial.println(bool_f1);
            //f2
            bool_f2 = ((strstr(get_arg, "f2=")+4)[0] == 't');
            Serial.print("F-2=");               
            Serial.println(bool_f2);
            //f3
            bool_f3 = ((strstr(get_arg, "f3=")+4)[0] == 't');
            Serial.print("F-3=");
            Serial.println(bool_f3);
            // ispolnenie
            set_lights();
          } 
 }
} //END loop
//End of sketch

